/**
 * EidoKropki -- Web-based SGF Editor for Kropki
 * Copyright (c) 2008,2009, Bartek Dyda <kropki@yahoo.co.uk>
 *
 * Most of EidoKropki code comes from EidoGo, a Web-based SGF Go Editor,
 * Copyright (c) 2007, Justin Kramer <jkkramer@gmail.com>
 *
 * Code licensed under AGPLv3:
 * http://www.fsf.org/licensing/licenses/agpl-3.0.html
 */

/**
 * @class Applies rules (capturing, ko, etc) to a board.
 */
eidogo.Rules = function(board) {
    this.init(board);
};
eidogo.Rules.prototype = {
    /**
     * @constructor
     * @param {eidogo.Board} board The board to apply rules to
     */
    init: function(board) {
        this.board = board;
        this.pendingCaptures = [];
	this.rules = "";
	this.punish       = false;  // punish for surrounding own dots
	this.mustsurround = false;  // Russian (automatic stops) or Polish (no automatic stops)
 	this.minimalarea  = false;
	this.lastdotsafe  = false;   // true is default for szkrab
    },
    getPunish: function() {
        return this.punish;
    },
    getMinimalArea: function() {
	return this.minimalarea;
    },
    getMustSurround: function() {
	return this.mustsurround;
    },
    getLastDotSafe: function() {
	return this.lastdotsafe;
    },
    getRule: function(rules, r) {
	if (rules.indexOf(r + '=0') >= 0)
	    return 0;
	if (rules.indexOf(r) >= 0)
	    return 1;
	return 0;
    },
    isRule: function(rules, r) {
	if (rules.indexOf(r) >= 0)
	    return true;
	return false;
    },
    setRules: function(newr) {
	this.rules = newr;
	newr = newr.toLowerCase();
	if (newr == '0' || newr == '1' || newr == '2'
	    || newr == '' || newr == 'punish') {   // program 'Kropki' rules, szkrab rules
	    if (newr=='0' || newr == '1' || newr == '2') {
		this.minimalarea = true;   // minimalarea=false was never (fully) implemented in Kropki
		if (newr=='0' || newr=='1')
		    this.mustsurround = true;
	    }
	    else 
		this.lastdotsafe = true;   // typical szkrab rule
	}
	else {    // rules in the new format
	    if (newr.indexOf('szkrab') >= 0)
		this.lastdotsafe = true;
	    if (newr.indexOf('russian') >= 0) {
		this.minimalarea = true;
		this.mustsurround = true;
	    }
	    this.punish = this.getRule(newr, 'punish');
	    if (this.isRule(newr, 'lastdotsafe'))
		this.lastdotsafe = this.getRule(newr, 'lastdotsafe');
	    if (this.isRule(newr, 'minimalarea'))
		this.minimalarea = this.getRule(newr, 'minimalarea');
	    if (this.isRule(newr, 'mustsurround'))
		this.mustsurround = this.getRule(newr, 'mustsurround');
	}
    },
    /**
     * Called to see whether a stone may be placed at a given point
    **/
    check: function(pt, color) {
        if (!this.board.isOnBoard(pt))
	    return false;     // not in board
        // already occupied?
        if (this.board.getStone(pt) != this.board.EMPTY) {
            return false;
        }
        // TODO: check for suicide? (allowed in certain rulesets)    
        // TODO: ko
        return true;
    },
    /**
     * Apply rules to the current game (perform any captures, etc)
    **/
    apply: function(pt, color) {
//        this.doCaptures(pt, color);
	if (this.mustsurround) {
	    this.board.doAutoStops(color);
	}
    }
}
